<?php
/**
 * Copyright (c) 2015 Jens Segers
 * The MIT License (MIT)
 * @laravel-admin-extensions/media-manager
 */

namespace App\Http\Controllers\Backend\Encore;

use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;
use League\Flysystem\Adapter\Local;

/**
 * Class MediaManager.
 */
class MediaManager
{
    /**
     * @var string
     */
    protected $path = '/';
    protected $disk;
    /**
     * @var \Illuminate\Filesystem\FilesystemAdapter
     */
    protected $storage;
    /**
     * @var array
     */
    protected $fileTypes = [
        'image' => 'png|jpg|jpeg|tmp|gif',
        'word'  => 'doc|docx',
        'ppt'   => 'ppt|pptx',
        'pdf'   => 'pdf',
        'code'  => 'php|js|java|python|ruby|go|c|cpp|sql|m|h|json|html|aspx',
        'zip'   => 'zip|tar\.gz|rar|rpm',
        'txt'   => 'txt|pac|log|md',
        'audio' => 'mp3|wav|flac|3pg|aa|aac|ape|au|m4a|mpc|ogg',
        'video' => 'mkv|rmvb|flv|mp4|avi|wmv|rm|asf|mpeg',
    ];
    /**
     * MediaManager constructor.
     *
     * @param string $path
     */
    public function __construct($disk = 'public', $path = '/')
    {
        $this->path = $path;
        $this->disk = $disk;
        $this->initStorage($disk);
    }
    private function initStorage($disk)
    {
        $this->storage = Storage::disk($disk);
        if (!$this->storage->getDriver()->getAdapter() instanceof Local) {
            return redirect('admin')->with('status', 'failed')->with('message', "Media only works for local storage");
        }
    }
    public function ls()
    {
        if (!$this->exists()) {
            return redirect('admin')->with('status', 'failed')->with('message', "File or directory [$this->path] not exists");
        }
        $files = $this->storage->files($this->path);
        $directories = $this->storage->directories($this->path);
        return $this->formatDirectories($directories)
            ->merge($this->formatFiles($files))
            ->sort(function ($item) {
                return $item['name'];
            })->all();
    }
    /**
     * Get full path for a giving fiel path.
     *
     * @param string $path
     *
     * @return string
     */
    protected function getFullPath($path)
    {
        return $this->storage->getDriver()->getAdapter()->applyPathPrefix($path);
    }
    public function download()
    {
        $fullPath = $this->getFullPath($this->path);
        if (File::isFile($fullPath)) {
            return response()->download($fullPath);
        }
        return response('', 404);
    }
    public function delete($path)
    {
        $paths = is_array($path) ? $path : func_get_args();
        foreach ($paths as $path) {
            $fullPath = $this->getFullPath($path);
            if (is_file($fullPath)) {
                $this->storage->delete($path);
            } else {
                $this->storage->deleteDirectory($path);
            }
        }
        return true;
    }
    public function move($new)
    {
        return $this->storage->move($this->path, $new);
    }
    /**
     * @param UploadedFile[] $files
     * @param string         $dir
     *
     * @return mixed
     */
    public function upload($files = [])
    {
        foreach ($files as $file) {
            $this->storage->putFileAs($this->path, $file, $file->getClientOriginalName());
        }
        return true;
    }
    public function newFolder($name)
    {
        $path = rtrim($this->path, '/').'/'.trim($name, '/');
        return $this->storage->makeDirectory($path);
    }
    public function exists()
    {
        $path = $this->getFullPath($this->path);
        return file_exists($path);
    }
    /**
     * @return array
     */
    public function urls()
    {
        return [
            'path'       => $this->path,
            'index'      => $this->disk == 'posts' ? route('backend.post-media-index') : route('backend.media-index'),
            'move'       => $this->disk == 'posts' ? route('backend.post-media-move') : route('backend.media-move'),
            'delete'     => $this->disk == 'posts' ? route('backend.post-media-delete') : route('backend.media-delete'),
            'upload'     => $this->disk == 'posts' ? route('backend.posts.upload.media') : route('backend.media-upload'),
            'new-folder' => $this->disk == 'posts' ? route('backend.post-media-new-folder') : route('backend.media-new-folder'),
        ];
    }
    public function formatFiles($files = [])
    {
        $files = array_map(function ($file) {
            return [
                'download'  => $this->disk == 'posts'? route('backend.post-media-download', compact('file')) : route('backend.media-download', compact('file')),
                'icon'      => '',
                'name'      => $file,
                'preview'   => $this->getFilePreview($file),
                'isDir'     => false,
                'size'      => $this->getFilesize($file),
                'link'      => $this->disk == 'posts' ? route('backend.post-media-download', compact('file')) : route('backend.media-download', compact('file')),
                'url'       => $this->storage->url($file),
                'time'      => $this->getFileChangeTime($file),
                'writable'      => is_writable($this->getFullPath($file)),
            ];
        }, $files);
        return collect($files);
    }
    public function formatDirectories($dirs = [])
    {
        $url = $this->disk == 'posts' ? route('backend.post-media-index', ['path' => '__path__', 'view' => request('view')]) : route('backend.media-index', ['path' => '__path__', 'view' => request('view')]);

        $pjax_class = $this->disk == 'posts' ? 'media-pjax' : 'pjax';

        $preview = "<a href=\"$url\" class=\"" . $pjax_class ."\"><span class=\"file-icon text-aqua\"><i class=\"fa fa-folder\"></i></span></a>";
        $dirs = array_map(function ($dir) use ($preview) {
            return [
                'download'  => '',
                'icon'      => '',
                'name'      => $dir,
                'preview'   => str_replace('__path__', $dir, $preview),
                'isDir'     => true,
                'size'      => '',
                'link'      => $this->disk == 'posts' ? route('backend.post-media-index', ['path' => '/'.trim($dir, '/'), 'view' => request('view')]) : route('backend.media-index', ['path' => '/'.trim($dir, '/'), 'view' => request('view')]),
                'url'       => $this->storage->url($dir),
                'time'      => $this->getFileChangeTime($dir),
                'writable'      => is_writable($this->getFullPath($dir)),
            ];
        }, $dirs);
        return collect($dirs);
    }
    public function navigation()
    {
        $folders = explode('/', $this->path);
        $folders = array_filter($folders);
        $path = '';
        $navigation = [];
        foreach ($folders as $folder) {
            $path = rtrim($path, '/').'/'.$folder;
            $navigation[] = [
                'name'  => $folder,
                'url'   => $this->disk == 'posts' ? route('backend.post-media-index', ['path' => $path]) : route('backend.media-index', ['path' => $path]),
            ];
        }
        return $navigation;
    }
    public function getFilePreview($file)
    {
        switch ($this->detectFileType($file)) {
            case 'image':
                if ($this->storage->getDriver()->getConfig()->has('url')) {
                    $url = $this->storage->url($file);
                    $preview = "<span class=\"file-icon has-img\" data-url=\"$url\"><img src=\"$url\" alt=\"Attachment\"></span>";
                } else {
                    $preview = '<span class="file-icon"><i class="fa fa-file-image"></i></span>';
                }
                break;
            case 'pdf':
                $preview = '<span class="file-icon"><i class="fa fa-file-pdf"></i></span>';
                break;
            case 'zip':
                $preview = '<span class="file-icon"><i class="fa fa-file-archive"></i></span>';
                break;
            case 'word':
                $preview = '<span class="file-icon"><i class="fa fa-file-word"></i></span>';
                break;
            case 'ppt':
                $preview = '<span class="file-icon"><i class="fa fa-file-powerpoint"></i></span>';
                break;
            case 'xls':
                $preview = '<span class="file-icon"><i class="fa fa-file-excel"></i></span>';
                break;
            case 'txt':
                $preview = '<span class="file-icon"><i class="fa fa-file-alt"></i></span>';
                break;
            case 'code':
                $preview = '<span class="file-icon"><i class="fa fa-code"></i></span>';
                break;
            default:
                $preview = '<span class="file-icon"><i class="fa fa-file"></i></span>';
        }
        return $preview;
    }
    protected function detectFileType($file)
    {
        $extension = File::extension($file);
        foreach ($this->fileTypes as $type => $regex) {
            if (preg_match("/^($regex)$/i", $extension) !== 0) {
                return $type;
            }
        }
        return false;
    }
    public function getFilesize($file)
    {
        $bytes = filesize($this->getFullPath($file));
        return $bytes;
    }

    public function getFileChangeTime($file)
    {
        $time = filectime($this->getFullPath($file));
        return date('Y-m-d H:i:s', $time);
    }
}